# frozen_string_literal: true

module Notifiable
  class NotificationStatus < ActiveRecord::Base
    self.table_name = 'notifiable_statuses'
    
    REJECTED_STATUS = 1
    SENT_STATUS = 2
    DELIVERED_STATUS = 3
    OPENED_STATUS = 4

    belongs_to :notification, class_name: 'Notifiable::Notification'
    validates :notification, presence: true

    belongs_to :device_token, class_name: 'Notifiable::DeviceToken'
    validates :device_token, presence: true

    def rejected!
      update_attributes(status: REJECTED_STATUS)
    end

    def sent!
      update_attributes(status: SENT_STATUS)
    end
    
    def delivered!
      update_attributes(status: DELIVERED_STATUS)
    end
    
    def delivered?
      status == DELIVERED_STATUS
    end
    
    def opened!
      update_attributes(status: OPENED_STATUS)
    end
    
    def opened?
      status == OPENED_STATUS
    end
  end
end
