# frozen_string_literal: true
require 'logger'

module Notifiable
  class NotifierBase    
    attr_reader :env, :notification

    def self.notifier_attribute(*vars)
      @notifier_attributes ||= []
      @notifier_attributes.concat vars
      attr_writer(*vars)
    end

    class << self
       attr_reader :notifier_attributes
     end

    def initialize(notification)
      @notification = notification
      @mutex = Mutex.new
    end

    def send_notification(device_token)
      enqueue(device_token, notification)
     end

    def close
      flush
      save_receipts if Notifiable.save_receipts
      @notification.save
    end

    protected
    
    def logger
      if defined?(Rails)
        Rails.logger
      else
        @logger ||= Logger.new(STDOUT)
      end
    end
 
    def flush; end

    def processed(device_token, status = NotificationStatus::SENT_STATUS, error_code = nil, error_message = nil)
      @mutex.synchronize {
        if @notification.app.save_notification_statuses
          receipts << { notification_id: notification.id, device_token_id: device_token.id, status: status, created_at: DateTime.now, error_code: error_code, error_message: error_message }
          save_receipts if receipts.count >= Notifiable.notification_status_batch_size
        end

        @notification.increment(:sent_count)
        @notification.increment(:gateway_accepted_count) if status == NotificationStatus::SENT_STATUS
        @notification.save if @notification.sent_count % Notifiable.notification_status_batch_size == 0
      }
    end

    private

    def receipts
      @receipts ||= []
    end

    def save_receipts
      Notifiable::NotificationStatus.import receipts, validate: false
      @receipts = []
    end
  end
end
